#!/usr/local/bin/perl
use Socket;

local($nextline);
local($ipproto,$fromhost,$fromport,$tohost,$toport,$size);
local($linkhdrlen);
local($linktypestr);

# from net/bpf.h of libpcap-0.3
local($DLT_NULL)=0;#       /* no link-layer encapsulation */
local($DLT_EN10MB)=    1;#       /* Ethernet (10Mb) */
local($DLT_EN3MB)=     2;#       /* Experimental Ethernet (3Mb) */
local($DLT_AX25)=      3;#       /* Amateur Radio AX.25 */
local($DLT_PRONET)=    4;#       /* Proteon ProNET Token Ring */
local($DLT_CHAOS)=     5;#       /* Chaos */
local($DLT_IEEE802)=   6;#       /* IEEE 802 Networks */
local($DLT_ARCNET)=    7;#       /* ARCNET */
local($DLT_SLIP)=      8;#       /* Serial Line IP */
local($DLT_PPP)=               9;#       /* Point-to-point Protocol */
local($DLT_FDDI)=      10;#      /* FDDI */
local($DLT_ATM_RFC1483)=       11;#      /* LLC/SNAP encapsulated atm */
local($DLT_RAW)=               12;#      /* raw IP */

# from linux-include/netinet/if_ether.h of tcpdump-3.3
local($ETHERTYPE_PUP)=           0x0200;#  /* PUP protocol */
local($ETHERTYPE_IP)=            0x0800;#  /* IP protocol */
local($ETHERTYPE_ARP)=           0x0806;#  /* Addr. resolution protocol */
local($ETHERTYPE_REVARP)=        0x8035;#  /* reverse Addr. resolution protocol */

# from /usr/include/linux/in.h of Linux2.0.x
local($IPPROTO_IP) = 0;#        /* Dummy protocol for TCP               */
local($IPPROTO_ICMP) = 1;#      /* Internet Control Message Protocol    */
local($IPPROTO_IGMP) = 2;#       /* Internet Group Management Protocol   */
local($IPPROTO_IPIP) = 4;#     /* IPIP tunnels (older KA9Q tunnels use 94) */
local($IPPROTO_TCP) = 6;#         /* Transmission Control Protocol        */
local($IPPROTO_EGP) = 8;#         /* Exterior Gateway Protocol            */
local($IPPROTO_PUP) = 12;#         /* PUP protocol                         */
local($IPPROTO_UDP) = 17;#         /* User Datagram Protocol               */
local($IPPROTO_IDP) = 22;#        /* XNS IDP protocol                     */
local($IPPROTO_RAW) = 255;#      /* Raw IP packets                       */


local($FDDIFC_LLC_ASYNC)= 0x50;#            /* Async. LLC frame */
local($FDDIFC_CLIFF) = 0xF0;#
local($LLC_UI)=          0x03;
local($LLCSAP_SNAP)=             0xaa;

sub printout
{
    if($proto eq ""){$proto = "other";}
    if($srcaddr ne ""){
	$srchost =inet_ntoa($srcaddr);$desthost=inet_ntoa($destaddr);
    }
    print "$proto,$srchost,$srcport,$desthost,$destport,$size\n";
}
sub getnextline
{
    $nextline=<>;
    if($nextline eq ""){
	return 0;
    }else{
	return 1;
    }
}
sub getonetcpdata
{
#    local(*ipproto,*fromhost,*fromport,*tohost,*toport,*size)=@_;
    $srchost = '';$desthost = '';
    $srcaddr ='';$destaddr ='';
    $networkproto = 0;$proto= '';
    $srcport = 0;$destport = 0;

    if(read(STDIN,$buf,16)<=0){return -1;}
#    ($tv_sec,$tv_usec,$caplen,$len) = unpack("LLLL",$buf);
    (undef,undef,$caplen,$size) = unpack("LLLL",$buf);
 #   $size = $len;
  #  print ":tv_sec=$tv_sec,tv_usec=$tv_usec,caplen=$caplen,len=$len\n";
    read(STDIN,$buf,$caplen);
    #printf("%d\n",unpack("n",substr($buf,12,2)));
    if($linktype==$DLT_IEEE802){
	$networkproto = unpack("n",substr($buf,12,2));
	#print ":networkproto=[$networkproto]\n";
	$buf = substr($buf,$linkhdrlen);
	$caplen -= $linkhdrlen;
    }elsif($linktype==$DLT_FDDI){
	if((unpack("C",substr($buf,0,1)) & $FDDIFC_CLIFF)==$FDDIFC_LLC_ASYNC){
	    $buf = substr($buf,13);
	    $caplen -= $linkhdrlen;
	    if(unpack("C",substr($buf,1,1))==$LLCSAP_SNAP
	       && unpack("C",substr($buf,0,1))==$LLCSAP_SNAP
	       && unpack("C",substr($buf,3,1))==$LLC_UI){
		$networkproto = unpack("n",substr($buf,6,2));
		$buf = substr($buf,8);
		$caplen -= $linkhdrlen;	
	    }    
	}
    }
   # printf("%d\n",unpack("C",substr($buf,14,1)));
   # print "linkhdrlen = $linkhdrlen\n";
    #printf("%d\n",unpack("C",substr($buf,0,1)));  
    if($networkproto == $ETHERTYPE_ARP){
	$proto = "arp";
    }elsif($networkproto == $ETHERTYPE_IP){
	($ver_ihl,$type,$iplen,$ident,$flagoffset,$lost,$transportproto,$checksum,$srcaddr,$destaddr) = unpack("CCnnnCCna4a4",$buf);
	$ver = $ver_ihl >> 4;
	$ihl = $ver_ihl & 0x0f;
	$flag = $flagoffset >>13;
	$offset = $flagoffset & 0x1fff;
	# print ":ver=$ver,ihl=$ihl,iplen=$iplen,ident=$ident,transportproto=$transportproto,caplen=$caplen,flag=$flag,offset=$offset\n";
	if($srcaddr eq ''){print ":srcaddr can't found\n";&printout;return 1;}
	if($destaddr eq ''){print ":destaddr can't found\n";&printout;return 1;}
	if($ver !=4){return 0;}
	$buf=substr($buf,$ihl*4);
	$caplen -= $ihl*4;
	if($transportproto == $IPPROTO_TCP){
	    $proto = "tcp";
	}elsif($transportproto == $IPPROTO_UDP){
	    $proto = "udp";
	}elsif($transportproto == $IPPROTO_ICMP){
	    $proto = "icmp";
	}else{
	    $proto = "ip";
	}
	if($transportproto == $IPPROTO_TCP || $transportproto==$IPPROTO_UDP){
	    ($srcport,$destport)=unpack("nn",$buf);
	    if(!($offset!=0 || ($flag & 1))){
		if($srcport==0){print ":warning:srcport=0\n";}
		if($destport==0){print ":warning:destport=0\n";}
	    }else{
#	    if($offset!=0 || ($flag & 1)){	# check flagmentation.
		#print "#################FLAGMENT(ident=$ident,offset=$offset,srcport=$srcport,destport=$destport)###############\n";
		$key = "$srcaddr:$destaddr:$transportproto:$ident";
		if($offset==0){
		    if($srcport==0 || $destport==0){
			print ":warning:srcport=0\n";
			&printout;return 1;
			# exit 1;
		    }
		    local($oldsize) = $size;
		    for($i=$#nextflagpacket;$i>=0;$i--){
			if($nextflagpacket[$i] eq $key){
			    @sizes = split(/:/,$nextflagpacket{$key});
			    # print "FOUND FLAGMENT PACKET\n";
			    foreach $s(@sizes){
				$size = $s;
				&printout;
			    }
			    delete $nextflagpacket{$key};
			    $nextflagpacket[$i]='';
			}
		    }
		    $size = $oldsize;

		    #print "###Adding.(srcport=$srcport,destport=$destport)\n";
		    unshift(@firstflagpacket,$key);
		    $firstflagpacket{$key} = "$srcport:$destport";
		    if($srcport == 0){print ":can't found srcport,\n";&printout;return 0;}
		    while(scalar(@firstflagpacket) > 10){
			delete $firstflagpacket{$#firstflagpacket};
			pop(@firstflagpacket);
		    }
		    #print "--firstflagpacket--\n";
		    #for($i=0;$i<@firstflagpacket;$i++){
			#print "[$i]",$firstflagpacket{$firstflagpacket[$i]},"\n";
		#    }

		}else{
		    $srcport=0;$destport=0;
		    #if($srcport!=0){
			#print "srcport!=0\n";
			#exit 1;
		    #}
		    for($i=0;$i<@firstflagpacket;$i++){
			if($firstflagpacket[$i] eq "$key"){
			    $srcportdestport = $firstflagpacket{$key};
			   # print "srcportdestport=[$srcportdestport],i=$i\n";
			    ($srcport,$destport) = split(/:/,$srcportdestport);
			   # print "FOUND firstflagpacket\n";
			    if($srcport == 0){print ":can't found srcport===.\n";return 1;}
			    last;
			}
		    }
		    if($srcport == 0){
			if(! defined($nextflagpacket{$key})){ 
			    unshift(@nextflagpacket,$key);
			}
			$nextflagpacket{$key} .= $size . ":";
			while(@nextflagpacket > 10){
			    delete $nextflagpacket{$nextflagpacket[$#$nextflagpacket]};	
			    pop(@nextflagpacket);
			}
			#print "NOT FOUND restoring!\n";
			return 0;
		    }
		  #  print "ERR:srcport!=0,srcport=[$srcport]\n";
		}
		#print "#################FLAGMENT END###############\n";

	    }
	}
    }elsif($networkproto == $ETHERTYPE_PUP){
	$proto = "pup";
    }elsif($networkproto == $ETHERTYPE_REVARP){
	$proto = "revarp";
    }
#    print "srcaddr=$srchost,dest=$desthost\n";
#    print "srcport=$srcport,destport=$destport\n";
    &printout;
    return 1;

    if($nextline eq ""){
	if(! &getnextline){return 0;}
    }
    $ipproto="none";
    $fromhost="";
    $tohost="";
    $fromport=0;
    $toport=0;
    $size=0;

    if($nextline =~ /^(\d\d):(\d\d):(\d\d)\.(\d+) arp (\S+) ([^\s\:]+) tell ([^\s\:]+)/){
	$ipproto="arp";
	$fromhost=$6;
	$fromport=$5;
	$tohost=$7;
   }elsif($nextline =~ /^(\d\d):(\d\d):(\d\d)\.(\d+) ([^\s\:]+) > ([^\s\:]+):/){
	$lost=$';
	$fromhost=$5;
	$tohost=$6;

	$ipproto='tcp';
	if($lost=~/^ (tcp|udp|icmp)/){
	    $ipproto=$1;
	}
	if($ipproto =~ /^(tcp|udp)$/){
	    
	    if($fromhost !~ /^\d+\.\d+\.\d+\.\d+$/ 
	       && $fromhost =~ /^(.+)\.([^\.]+)/){
		$fromhost=$1;
		$fromport=$2;
	    }
	    if($tohost !~ /^\d\.\d\.\d\.\d$/
	       && $tohost =~ /^(.+)\.([^\.]+)/){
		$tohost=$1;
		$toport=$2;
	    }
	}
	if($lost =~ /nfs/){
	    $toport="nfs";
	}
    }else{
	$ipproto="other";
    }
   $size=0;
    while(1){
	&getnextline;
	if($nextline =~ /^\s+[0-9a-z]+\s+([0-9a-z]+)\s+/){
	    if($size==0 && $ipproto ne "other"){
		$size=hex($1);
	    }
	}else{
	    return 1;
	}
    }
}

#main
{
    $|=1;
    $ret=1;

    read(STDIN,$buf,24);
    ($magic,$ver_major,$ver_minor,$timezone,$sigfigs,$snaplen,$linktype)
	= unpack("LSSlLLL",$buf);
    printf(":header=(%x,%d,%d,%d,%d,%d,%d)\n",$magic,$ver_major,$ver_minor,$timezone,$sigfigs,$snaplen,$linktype);
    if($magic != 0xa1b2c3d4){
	print "TCPDUMP MAGIC CODE ERROR\n";
	exit 1;
    }
    if($linktype == $DLT_EN10MB || $linktype == $DLT_EN3MB){
	$linktype = $DLT_IEEE802;
    }
    @LINKHDRLEN = (4,14,14,-1,-1,-1,14,-1,16,4,13,28,0);# from net/bpf.h of libpcap
    $linkhdrlen = $LINKHDRLEN[$linktype];
    print ":linkhdrlen=[$linkhdrlen]\n";
    if($linktype!=$DLT_IEEE802 && $linktype!=$DLT_FDDI){
	print "Not suppoted linktype[$linktype](current support=Ether).please send me email.\n";
    }
    if($linktype == $DLT_FDDI){
	print "You use FDDI! FDDI is now test implemented.please email to me.\n";
    }
    while($ret!=-1){
#    $ret=&getonetcpdata(*ipproto,*fromhost,*fromport,*tohost,*toport,*size);
        $ret=&getonetcpdata;
#	print "$ipproto,$fromhost,$fromport,$tohost,$toport,$size\n";
	#if($ret==1){
	#    print "$proto,$srchost,$srcport,$desthost,$destport,$size\n";
	#}
    }
    print STDERR "End of file!\n";
}








